﻿BetterTooltips = LibStub("AceAddon-3.0"):NewAddon("BetterTooltips", "AceHook-3.0");

-- GLOBAL VARIABLES
bttConfigCombat = false;

-- LOCAL FUNCTIONS AND HOOKS
local function eventHandler(self, event, ...)
	if ( event == "PLAYER_ENTERING_WORLD" ) then
		self:UnregisterEvent("PLAYER_ENTERING_WORLD")
		DEFAULT_CHAT_FRAME:AddMessage(string.format("|cffcc9900BetterTooltips 2.0 for WoD has loaded!|r"))
	end
end

local frame = CreateFrame("Frame")
frame:RegisterEvent("PLAYER_ENTERING_WORLD")
frame:SetScript("OnEvent", eventHandler)

local function levelcolor(level)
	if level then
		local color = GetQuestDifficultyColor(level)
		return string.format("%02x%02x%02x", color.r*255, color.g*255, color.b*255)
	end
end

local colors = {}
for class,color in pairs(RAID_CLASS_COLORS)
	do colors[class] = string.format("%02x%02x%02x", color.r*255, color.g*255, color.b*255);
end

function BetterTooltips:OnInitialize()
	-- do nothing
end

function BetterTooltips:OnEnable()
	self:SecureHookScript(FriendsMicroButton, "OnEnter", "OnMouseOverFriends");
	self:SecureHookScript(CharacterMicroButton, "OnEnter", "OnMouseOverCharacter");
	self:SecureHookScript(SpellbookMicroButton, "OnEnter", "OnMouseOverSpellbook");
	self:SecureHookScript(GuildMicroButton, "OnEnter", "OnMouseOverGuild");
end

function BetterTooltips:OnDisable()
	self:UnHookAll();
end

-- SOCIAL TOOLTIP
function BetterTooltips:OnMouseOverFriends()
	if ((UnitAffectingCombat("player")) and (bttConfigCombat)) then
		-- Do Nothing
	else
		local numBNet, numBNetOnline = BNGetNumFriends();
		local numFriends, numFriendsOnline = GetNumFriends();
		local fcolor = "ffffff";
		GameTooltip:AddLine(" ");
		
		-- Battle.net Friends
		if numBNetOnline > 0 then
			fcolor = "00ff00";
			if numBNetOnline == 1 then
				GameTooltip:AddLine("|cff"..fcolor.."1 Battle.net Friend Online|r");
			else
				GameTooltip:AddLine("|cff"..fcolor..string.format("%d Battle.net Friends Online", numBNetOnline).."|r");
			end
		else
			GameTooltip:AddLine("|cff"..fcolor.."No Battle.net Friends Online|r");
		end
		for i = 1, numBNet, 1 do
			local presenceID, realName, _, _, toonName, toonID, client, isOnline = BNGetFriendInfo(i);
			if isOnline then
				local zone = client;
				if client == "WoW" then
					local _, _, _, zoneName = BNGetToonInfo(presenceID);
					zone = zoneName;
				end
				GameTooltip:AddLine("|cff00fbf6"..toonName.." ("..zone..") ["..realName.."]|r");
			end
		end
		
		-- Spacer Line
		if (numFriendsOnline > 0) or (numBNetOnline > 0) then
			GameTooltip:AddLine(" ");
		end
		
		-- Regular Friends
		if numFriendsOnline > 0 then
			fcolor = "00ff00";
			if numFriendsOnline == 1 then
				GameTooltip:AddLine("|cff"..fcolor.."1 WoW Friend Online|r");
			else
				GameTooltip:AddLine("|cff"..fcolor..string.format("%d WoW Friends Online", numFriendsOnline).."|r");
			end
		else
			fcolor = "ffffff";
			GameTooltip:AddLine("|cff"..fcolor.."No WoW Friends Online|r");
		end
		for i = 1, numFriends, 1 do
			local name, level, class, area, connected, status, note = GetFriendInfo(i);
			if connected then
					GameTooltip:AddDoubleLine(("|cff"..levelcolor(level)..string.format("%2d", level).."|r".."   "..string.format("|cff%s%s", colors[class:gsub(" ", ""):upper()] or "ffffff", name).."|r"),(area));
			end
		end
		
		GameTooltip:Show();
	end
end

-- CHARACTER INFO TOOLTIP
function BetterTooltips:OnMouseOverCharacter()
	if ((UnitAffectingCombat("player")) and (bttConfigCombat)) then
		-- Do Nothing
	else
		--Durability
		local itemSlots = {
		"HeadSlot",
		"ShoulderSlot",
		"ChestSlot",
		"WristSlot",
		"HandsSlot",
		"WaistSlot",
		"LegsSlot",
		"FeetSlot",
		"MainHandSlot",
		"SecondaryHandSlot",
		}
		if not GetNumSubgroupMembers then
			tinsert(itemSlots, "RangedSlot")
		end
		local durability, maximum = 0, 0
		for i=1, #itemSlots do
			local slotId = GetInventorySlotInfo(itemSlots[i])
			local d, m = GetInventoryItemDurability(slotId)
			durability = durability + (d or 0)
			maximum = maximum + (m or 0)
		end
		local duraLast = durability / maximum * 100;
		local duraColor = "ffffff";
		if duraLast > 74 then duraColor = "00ff00"
			elseif duraLast > 59 then duraColor = "9acd32"
			elseif duraLast > 39 then duraColor = "ffff00"
			elseif duraLast > 19 then duraColor = "ffa500"
			elseif duraLast > 9 then duraColor = "ff4500"
			else duraColor = "ff0000"
		end	
		GameTooltip:AddDoubleLine(("Durability: "),(("|T%s:0|t"):format("Interface\\Icons\\Trade_BlackSmithing").."  |cff"..duraColor..("%.0f%%"):format(duraLast).."|r"));
		GameTooltip:AddLine(" ");
		--Money
		local money = GetCoinTextureString(GetMoney("player"))
		GameTooltip:AddDoubleLine(("Money: "),("|cffffffff"..money.."|r"))
		GameTooltip:AddLine(" ");
		--Currency
		GameTooltip:AddLine("Tracked Currency");
		local numCurTypes = GetCurrencyListSize()
		for i = 1, numCurTypes, 1 do
			local name, isHeader, isExpanded, isUnused, isWatched, count, icon = GetCurrencyListInfo(i);
			if isWatched then
				GameTooltip:AddDoubleLine("|cffffffff"..name.."|r","|cffffffff"..("%d%s|T%s:0|t"):format(count,"  ",icon).."|r");
			end
		end
		GameTooltip:Show();
	end
end

-- SPELLBOOK & ABILITIES TOOLTIP
local function AddSkillToTooltip(skillid, fallback)
	if (not skillid) then
		return GameTooltip:AddDoubleLine(fallback, "See Trainer!");
	end
	local name, icon, skillLevel, maxSkillLevel, numAbilities, spelloffset, skillLine = GetProfessionInfo(skillid);
	GameTooltip:AddDoubleLine("|cffffffff"..("|T%s:0|t%s"):format(icon, "   "..name).."|r", ("%d/%d"):format(skillLevel, maxSkillLevel));
end

function BetterTooltips:OnMouseOverSpellbook()
	if (UnitAffectingCombat("player") and bttConfigCombat) then
		-- Do Nothing
	else
		local prof1, prof2, arch, fish, cook, firstAid = GetProfessions();
		GameTooltip:AddLine(" ");	
		GameTooltip:AddLine("Professions");
		AddSkillToTooltip(prof1, PROFESSIONS_FIRST_PROFESSION);
		AddSkillToTooltip(prof2, PROFESSIONS_SECOND_PROFESSION);
		AddSkillToTooltip(arch, PROFESSIONS_ARCHAEOLOGY);
		AddSkillToTooltip(fish, PROFESSIONS_FISHING);
		AddSkillToTooltip(cook, PROFESSIONS_COOKING);
		AddSkillToTooltip(firstAid, PROFESSIONS_FIRST_AID);
		GameTooltip:Show();
	end
end

-- GUILD TOOLTIP
function BetterTooltips:OnMouseOverGuild()
	if (UnitAffectingCombat("player") and bttConfigCombat) then
		-- Do Nothing
	else
		if IsInGuild() then
			GuildRoster();
			local guildName, guildRankName, guildRankIndex = GetGuildInfo("player");
			local numGuildMembers, numOnline = GetNumGuildMembers();
			local sMsg = "";
			local spacer = true;
			if (numOnline) == 1 then
				sMsg = "1 Guildie Online";
			else
				sMsg = string.format("%0d Guildies Online", numOnline);
			end
			GameTooltip:AddLine("|cffffffff"..guildName.." - |r".."|cff00ff00"..sMsg.."|r");

			for i = 1, numOnline, 1 do
				local name, rank, rankIndex, level, class, zone, note, officernote, online, status, classFileName, achievementPoints, achievementRank, isMobile = GetGuildRosterInfo(i);
				if online then
					if (level < 100) then
						GameTooltip:AddDoubleLine(("|cff"..levelcolor(level).."  "..string.format("%2d", level).."|r".."   "..string.format("|cff%s%s", colors[classFileName] or "ffffff", name).."|r"),(zone));
					else
						GameTooltip:AddDoubleLine(("|cff"..levelcolor(level)..string.format("%2d", level).."|r".."   "..string.format("|cff%s%s", colors[classFileName] or "ffffff", name).."|r"),(zone));
					end
				end
			end
			
			for i = 1, numGuildMembers, 1 do
				local name, rank, rankIndex, level, class, zone, note, officernote, online, status, classFileName, achievementPoints, achievementRank, isMobile = GetGuildRosterInfo(i);
				if isMobile then
					if (spacer) then
						GameTooltip:AddLine(" ");
						GameTooltip:AddLine("Remote Guildies (Mobile Armory App)");
						spacer = false;
					end
					if (level < 100) then
						GameTooltip:AddLine(("|cff"..levelcolor(level).."  "..string.format("%2d", level).."|r".."   "..string.format("|cff%s%s", colors[classFileName] or "ffffff", name).."|r"));
					else
						GameTooltip:AddLine(("|cff"..levelcolor(level)..string.format("%2d", level).."|r".."   "..string.format("|cff%s%s", colors[classFileName] or "ffffff", name).."|r"));
					end
				end
			end
			
		else
			GameTooltip:AddLine("You are not currently in a guild.");
			GameTooltip:AddLine(" ");
			GameTooltip:AddLine("Once you join a guild, information about your guild mates who are logged on will appear in this tooltip.");
		end
		GameTooltip:Show();
	end
end

-- SLASH COMMANDS
SLASH_BTTCONFIG1, SLASH_BTTCONFIG2 = "/betterTooltips", "/btt";
function SlashCmdList.BTTCONFIG(arg)
	-- Combat Visibility Toggle
	if (arg == "combat") then
		-- Toggle and echo current setting
		bttConfigCombat = (bttConfigCombat == false);
		if (bttConfigCombat) then
			DEFAULT_CHAT_FRAME:AddMessage("|cffcc9900BetterTooltips will be hidden during combat|r");
		else
			DEFAULT_CHAT_FRAME:AddMessage("|cffcc9900BetterTooltips will show both in and out of combat|r");
		end
	else
		-- Print help
		DEFAULT_CHAT_FRAME:AddMessage("|cffcc9900Better Tooltips Slash Command Help|r");
		DEFAULT_CHAT_FRAME:AddMessage("|cffcc9900Use /BetterTooltips or /btt with a command:|r");
		DEFAULT_CHAT_FRAME:AddMessage("|cffcc9900/btt combat - Hide Better Tooltips during combat.|r");
	end
end
